@@ -26,10 +26,10 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: mgp.c,v 1.117 2001/05/24 09:16:04 kazu Exp $
+ * $Id: mgp.c,v 1.136 2004/09/14 14:48:47 nishida Exp $
  */
 
-static char *mgp_version = "1.08a (20010524)";
+static char *mgp_version = "1.11a (20040915)";
 
 #include "mgp.h"
 #ifdef USE_SETLOCALE
@@ -196,28 +196,66 @@ main(argc, argv)
 	int argc;
 	char **argv;
 {
+	int i, j;              /* counters */
+	int tmp_argc;          /* number of current arguments */
+	char **tmp_argv;       /* manipulated 'argv' */
 	int opt;
 	extern char *optarg;
 	extern int optind;
 	char *progname;
-	u_int start_page = 1;
+	int start_page = 1;
 	char buf[BUFSIZ], *p, *p2;
 	extern char *Paths[];
 	extern int NumPaths;
 #ifdef VFLIB
 	char *vfont_name = VFONT;
 #endif
+	char *ep;
 
 #ifdef USE_SETLOCALE
 	setlocale(LC_CTYPE, "");
 #endif
 	progname = argv[0];
 
+	/* secure by default.  If you need fork/exec, use -U */
+	mgp_flag |= FL_NOFORK;
+
 	if ((p = getenv("MGPWDIR")) != NULL) {
 		mgpwdir = p;
 	}
 
-#define ACCEPTOPTS	"BdvVob:c:eg:f:hlGp:qt:Q:PST:D:CORw:X:x:nF:E:"
+	/*
+	 * check for the argument '--title' to set the window's title:
+	 * go through the whole 'argv' and cut off this option since it
+	 * will not be accepted by getopt()
+	 * default title is 'MagicPoint'
+	 */
+	tmp_argv=(char**)malloc(argc*sizeof(char*));
+	tmp_argc=argc;
+	i=j=0;
+	while (i < argc)
+	{
+		if (strcmp(argv[i], "--title") == 0)
+		{
+			tmp_argc--;
+			if (++i < argc)
+			{
+				mgp_wname=strdup(argv[i]);
+				tmp_argc--;
+			}
+		}
+		else
+			tmp_argv[j++]=strdup(argv[i]);
+		i++;
+	}
+	/* set title to default if not set by user */
+	if (argc-1 <= tmp_argc)
+		mgp_wname=strdup("MagicPoint");
+
+	argv=tmp_argv;
+	argc=tmp_argc;
+
+#define ACCEPTOPTS	"Bd:vVob:c:eg:f:hlGp:qt:Q:PSUT:D:CORw:X:x:nF:E:"
 	while ((opt = getopt(argc, argv, ACCEPTOPTS)) != -1) {
 #undef ACCEPTOPTS
 		switch (opt) {
@@ -227,6 +265,8 @@ main(argc, argv)
 
 		case 'd':
 			mgp_flag |= FL_DEMO;
+			if (isdigit(optarg[0])) demointerval = atoi(optarg);
+			else optind --;
 			break;
 
 		case 'V':
@@ -261,7 +301,12 @@ main(argc, argv)
 			break;
 
 		case 'p':
+			ep = NULL;
 			start_page = atoi(optarg);
+			if (start_page <= 0) {
+				mgp_usage(progname);
+				/*NOTREACHED*/
+			}
 			break;
 
 		case 'e':
@@ -294,6 +339,9 @@ main(argc, argv)
 		case 'S':
 			mgp_flag |= FL_NOFORK;
 			break;
+		case 'U':
+			mgp_flag &= ~FL_NOFORK;
+			break;
 
 		case 'T':
 			tsfile = optarg;
@@ -328,6 +376,10 @@ main(argc, argv)
 				mgp_flag |= FL_NOVFLIB;
 			else if (strcmp(optarg, "freetype") == 0)
 				mgp_flag |= FL_NOFREETYPE;
+			else if (strcmp(optarg, "xft") == 0)
+				mgp_flag |= FL_NOXFT;
+			else if (strcmp(optarg, "m17n") == 0)
+				mgp_flag |= FL_NOM17N;
 			else {
 				fprintf(stderr, "unknown rendering engine %s\n",
 					optarg);
@@ -388,7 +440,7 @@ main(argc, argv)
     }
 
 	init_win1(xgeometry);
-	strncpy(buf, mgp_fname, sizeof(buf));
+	strlcpy(buf, mgp_fname, sizeof(buf));
 	if ((p = rindex(buf, '/'))) {
 		*p = '\0';
         	Paths[NumPaths++]= expandPath(buf);
@@ -440,7 +492,7 @@ genhtml(start_page)
 	FILE *txt;
 	int page;
 	char *childdebug;
-	char convdb[][3][16] = {{ "jpg", "cjpeg", "djpeg" },
+	char *convdb[][3] = {{ "jpg", "cjpeg", "djpeg" },
 				{ "png", "pnmtopng", "pngtopnm" },
 				{ NULL, NULL, NULL }};
 	int inum = 0;
@@ -453,6 +505,12 @@ genhtml(start_page)
 		}
 		if (*convdb[inum] == NULL) {
 			fprintf(stderr, "unknwon image type %s.\n", htmlimage);
+			/* print out valid image types */
+			fprintf(stderr, "Valid image types: ");
+			for (inum = 0; *convdb[inum] != 0; inum++) {
+				fprintf(stderr, "%s ", *convdb[inum] );
+			}
+			fprintf(stderr,"\n");
 			cleanup(-1);
 		}
 	}
@@ -487,13 +545,19 @@ genhtml(start_page)
 		sprintf(buf, "xwintoppm -silent -name MagicPoint | "
 			"%s %s > %s/mgp%05d.%s",
 			convdb[inum][1], childdebug, dumpdir, page, EXT);
-		system(buf);	/*XXX security hole*/
+		if (system(buf) < 0){	/*XXX security hole*/
+			fprintf(stderr, "system() failed for %s", buf);
+			exit(-1);
+		}
 		fprintf(stderr, "(thumbnail)");
 		sprintf(buf, "%s %s/mgp%05d.%s | "
 			"pnmscale 0.25 | %s %s > %s/mgp%05d.idx.%s",
 			convdb[inum][2], dumpdir, page, EXT, convdb[inum][1], childdebug,
 			dumpdir, page, EXT);
-		system(buf);	/*XXX security hole*/
+		if (system(buf) < 0){	/*XXX security hole*/
+			fprintf(stderr, "system() failed for %s", buf);
+			exit(-1);
+		}
 
 		/*
 		 * dump out html file
@@ -537,7 +601,7 @@ genhtml(start_page)
 				page);
 		}
 		fprintf(html, "<HR>Generated by "
-		    "<A HREF=\"http://www.mew.org/mgp/\">MagicPoint</A>\n"
+		    "<A HREF=\"http://member.wide.ad.jp/wg/mgp/\">MagicPoint</A>\n"
 		    "</BODY></HTML>\n");
 		fclose(html);
 
@@ -606,7 +670,7 @@ txtdone:;
 	}
 	fprintf(html, "<HR>\n");
 	fprintf(html, "Generated by "
-		"<A HREF=\"http://www.mew.org/mgp/\">"
+		"<A HREF=\"http://member.wide.ad.jp/wg/mgp/\">"
 		"MagicPoint</A>\n");
 	fprintf(html, "<BR>\n</BODY></HTML>\n");
 	fclose(html);
@@ -641,7 +705,7 @@ mgp_usage(name)
 	fprintf(stderr, "\t-f <font>: Specify default vflib font\n");
 #endif /*VFLIB*/
 	fprintf(stderr, "\t-g <geometry>: Set window geometry\n");
-	fprintf(stderr, "\t-h Display this help message\n");
+	fprintf(stderr, "\t-h: Display this help message\n");
 #ifdef VFLIB
 	fprintf(stderr, "\t-l: Do not use outline font on vflib\n");
 #endif /*VFLIB*/
@@ -657,15 +721,19 @@ mgp_usage(name)
 	fprintf(stderr, "\t-B: Ignore background image\n");
 	fprintf(stderr, "\t-C: Use private colormap\n");
 	fprintf(stderr, "\t-D <dir>: Generate html pages for the presentation\n");
+	fprintf(stderr, "\t-E <name>: Use this image format in html (jpg or png)\n");
 	fprintf(stderr, "\t-F<mode>,<effect>,<value>: Use forwarding caches\n");
 	fprintf(stderr, "\t-G: Page guide is on\n");
 	fprintf(stderr, "\t-O: Obey to the window manager\n");
 	fprintf(stderr, "\t-Q <quality>: Set background image quality(0-100)\n");
 	fprintf(stderr, "\t-R: Do not perform automatic reload\n");
-	fprintf(stderr, "\t-S: Do not process directives that forks process\n");
+	fprintf(stderr, "\t-S: Do not process directives that forks process (default)\n");
+	fprintf(stderr, "\t-U: Do process directives that forks process (unsecure mode)\n");
 	fprintf(stderr, "\t-T <timestampfile>: Update timestampfile on page refresh\n");
+	fprintf(stderr, "\t-P: print stderr from image conversion tools (by default it's discarded)\n");
 	fprintf(stderr, "\t-V: Be verbose\n");
 	fprintf(stderr, "\t-X <gsdevice>: ghostscript device to use\n");
+	fprintf(stderr, "\t--title <title>: Set window title\n");
 
 	exit(0);
 }
@@ -1034,6 +1102,10 @@ repaint:;
 						0, 0, window_width, window_height, 0, 0);
 				break;
 
+			case XK_w:
+				toggle_fullscreen();		
+				break;
+
 			default:
 				number = 0;
 				break;
@@ -1057,6 +1129,10 @@ repaint:;
 			}
 			break;
 
+#if 1  /* by h.kakugawa@computer.org */
+		case MapNotify:
+		  goto repaint;
+#endif
 		case Expose:
 			if (e.xexpose.window != window) 
 				break;
@@ -1081,12 +1157,18 @@ repaint:;
 			break;
 
 		case ConfigureNotify:
-			if (e.xconfigure.window != window)
+			if ((e.xconfigure.window != window) && 
+				((mgp_flag & FL_OVER) || 
+				e.xconfigure.window != RootWindow(display, screen)))
 				break;
 			if (window_width != e.xconfigure.width
 			 || window_height != e.xconfigure.height) {
 				struct ctrl *lastcp;
 
+				if (!(mgp_flag & FL_OVER))
+					XMoveResizeWindow(display, window, 0, 0, 
+						e.xconfigure.width, e.xconfigure.height);
+
 				if (pg_mode)
 					pg_off();
 				pl_off(&state);
